/* Metadata parsing
 *
 * Karas are stored in matroska (mkv) files. Specs here:
 * - EBML (like binary XML or something):
 *   https://github.com/cellar-wg/ebml-specification/blob/master/specification.markdown
 * - Matroska EBML structure:
 *   https://www.matroska.org/technical/specs/index.html
 * - Matroska tags:
 *   https://www.matroska.org/technical/specs/tagging/index.html
 *
 * Kara metadata is saved as followed:
 *
 *   +--------------+--------------+-----------------+
 *   |   Metadata   | matroska tag | TargetTypeValue |
 *   +--------------+--------------+-----------------+
 *   | song_name    | TITLE        |   50            |
 *   | source_name  | TITLE        |   70            |
 *   | category     | CONTENT_TYPE |   50            |
 *   | language     | ADDRESS      |   50            |
 *   | author_name  | ARTIST       |   50            |
 *   | song_type    | GENRE        |   50            |
 *   +--------------+--------------+-----------------+
 */
#pragma once

#if defined(__cplusplus)
extern "C" {
#endif

#include <lektor/common.h>
#include <lektor/database.h>

struct kara_metadata {
    char song_name[LEKTOR_TAG_MAX];
    char source_name[LEKTOR_TAG_MAX];
    char category[LEKTOR_TAG_MAX];
    char language[LEKTOR_TAG_MAX];
    char author_name[LEKTOR_TAG_MAX];
    char song_type[LEKTOR_TAG_MAX];
    int song_number;
};

struct kara_metadata_rules {
    char category_rules[LKT_LINE_MAX];
    char language_rules[LKT_LINE_MAX];
    char song_type_rules[LKT_LINE_MAX];
};

void kara_metadata_rules_from_config(struct kara_metadata_rules *, lkt_db *);

/* Reads the .mkv file at `filename` and stores its metadata in `dst`.
   Returns 0 on success and -1 on error. */
int kara_metadata_read(struct kara_metadata *dst, const char *filename);

/* Reads the duration of the .mkv file `filename`. Returns 0 on success
   and -1 on error. */
int kara_read_length(double *len, const char *filename);

/* Write metadata to a mkv file. Returns 0 on success and -1 on error */
int kara_metadata_write(struct kara_metadata *mdt, const char *filename);

/* Concatenate the mdt in a string that is allocated and the user must free it */
void mdtcat(struct kara_metadata *mdt, char **ret);

/* Set the metadata for the file according to its path. */
int metadata_set_file(char *karapath, lkt_db *db);

/* Set metadata for files under a given directory by their path. */
int metadata_set_directory(const char *kara_dir, lkt_db *db);

/* Returns 0 if the correct magic number is found in the file, anything
 * otherwise. The correct magic number is `1A 45 DF A3`. */
int mkv_check_magic(const char *file);

/* Returns 0 if the correct magic number in `tab`, anything otherwise. The
 * correct magic number is `1A 45 DF A3`. */
int mkv_magic(uint8_t tab[4]);

#if defined(__cplusplus)
}
#endif
