use crate::{constants, utils, widgets::render_action_menu};
use amadeus_lib::actions;
use amadeus_macro::either;
use lkt_lib::*;

/// A simple card trait
pub trait Card<'a, LktType: LektorType<'a>>: ToString + Clone {
    /// Create an instance of the card from the lektor type
    fn new(lkt_type: LktType) -> Self;

    /// Render the card to the screen
    fn render(&mut self, ui: &mut egui::Ui, dark_mode: bool, actions: &[actions::Action]);

    /// Render the card to the screen. Variant to try to render the element in a
    /// more compact way.
    fn render_compact(&self, ui: &mut egui::Ui, dark_mode: bool);

    /// Returns the Ids of the activated actions.
    fn fulfilled_actions(&mut self) -> Vec<actions::Action>;

    /// Get the unique id from the inner lkt type.
    fn unique_id(&self) -> u64;

    /// The card can indicate if it need to be separator by separators
    const NEED_SEPARATOR: bool;

    /// Indicate if there is a need to have a separator between the header and
    /// the first row.
    const NEED_HEADER_SEPARATOR: bool;

    /// Set the spacing to insert after the last card is rendered.
    const BOTTOM_SPACE: Option<f32>;
}

#[derive(Clone)]
pub struct KaraCard {
    pub inner: Kara,
    actions: Vec<actions::Action>,
}

#[derive(Clone)]
pub struct PlaylistCard {
    pub inner: Playlist,
    actions: Vec<actions::Action>,
}

impl ToString for KaraCard {
    fn to_string(&self) -> String {
        format!(
            "{} - {} / {} - {}{} - {} [{}]{}",
            self.inner.category,
            self.inner.language,
            self.inner.source_name,
            self.inner.song_type,
            match self.inner.song_number {
                Some(num) => num.to_string(),
                None => "".to_string(),
            },
            self.inner.title,
            self.inner.author,
            either!(self.inner.is_available => ""; " (unavailable)")
        )
    }
}

impl ToString for PlaylistCard {
    fn to_string(&self) -> String {
        self.inner.name.clone()
    }
}

impl AsRef<str> for PlaylistCard {
    fn as_ref(&self) -> &str {
        &self.inner.name
    }
}

impl Card<'_, Kara> for KaraCard {
    const NEED_SEPARATOR: bool = true;
    const NEED_HEADER_SEPARATOR: bool = true;
    const BOTTOM_SPACE: Option<f32> = Some(constants::BOTTOM_PANEL_MAX_SIZE);

    fn new(inner: Kara) -> Self {
        Self {
            inner,
            actions: Vec::with_capacity(5),
        }
    }

    fn render(&mut self, ui: &mut egui::Ui, dark_mode: bool, actions: &[actions::Action]) {
        ui.add_space(constants::PADDING);
        static MIN_WIDTH_FOR_ADDITIONAL_INFOS: f32 = 1024.;
        let song = format!(
            "{} - {}{} - {}",
            self.inner.source_name,
            self.inner.song_type,
            match self.inner.song_number {
                Some(num) => num.to_string(),
                None => "".to_string(),
            },
            self.inner.title
        );
        ui.horizontal(|ui| {
            ui.add_space(constants::PADDING * 3.);
            let left_space = ui.available_width();
            ui.vertical(|ui| {
                ui.horizontal(|ui| {
                    ui.style_mut().override_text_style = Some(utils::font::body());
                    ui.colored_label(
                        constants::get_text_color(dark_mode),
                        format!("{} - {}", self.inner.category, self.inner.language),
                    );
                    if left_space >= MIN_WIDTH_FOR_ADDITIONAL_INFOS {
                        ui.with_layout(egui::Layout::right_to_left(egui::Align::Min), |ui| {
                            ui.label(format!("kara by {}", self.inner.author));
                        });
                    }
                });
                ui.horizontal(|ui| {
                    ui.style_mut().override_text_style = Some(utils::font::heading2());
                    render_action_menu(ui, actions, "▶", &mut self.actions);
                    ui.colored_label(constants::get_text_color(dark_mode), song);
                    ui.style_mut().override_text_style = Some(utils::font::body());
                    if left_space >= MIN_WIDTH_FOR_ADDITIONAL_INFOS {
                        ui.with_layout(egui::Layout::right_to_left(egui::Align::Min), |ui| {
                            if !self.inner.is_available {
                                ui.colored_label(
                                    constants::get_accent_color(dark_mode),
                                    " (unavailable)",
                                );
                            }
                            ui.label(format!(" #{}", self.inner.id));
                        });
                    }
                });
            });
        });
        ui.add_space(constants::PADDING);
    }

    fn render_compact(&self, ui: &mut egui::Ui, dark_mode: bool) {
        let header = format!(
            "{} - {} by {}",
            self.inner.category, self.inner.language, self.inner.author
        );
        let song = format!(
            "{} - {}{} - {}",
            self.inner.source_name,
            self.inner.song_type,
            match self.inner.song_number {
                Some(num) => num.to_string(),
                None => "".to_string(),
            },
            self.inner.title
        );
        ui.horizontal(|ui| {
            ui.add_space(constants::PADDING * 3.);
            ui.vertical(|ui| {
                ui.add_space(constants::PADDING * 2.);
                ui.horizontal(|ui| {
                    ui.style_mut().override_text_style = Some(utils::font::heading3());
                    ui.colored_label(constants::get_text_color(dark_mode), header);
                    ui.style_mut().override_text_style = Some(utils::font::body());
                    ui.label(egui::RichText::new(format!("#{}", self.inner.id)));
                });
                ui.horizontal(|ui| {
                    ui.style_mut().override_text_style = Some(utils::font::heading1());
                    ui.colored_label(constants::get_text_color(dark_mode), song);
                    if !self.inner.is_available {
                        ui.colored_label(constants::get_accent_color(dark_mode), " (unavailable)");
                    }
                });
            });
        });
    }

    fn fulfilled_actions(&mut self) -> Vec<actions::Action> {
        let ret = self.actions.clone();
        self.actions.clear();
        ret
    }

    fn unique_id(&self) -> u64 {
        self.inner.unique_id()
    }
}

impl Card<'_, Playlist> for PlaylistCard {
    const NEED_SEPARATOR: bool = false;
    const NEED_HEADER_SEPARATOR: bool = false;
    const BOTTOM_SPACE: Option<f32> = None;

    fn new(inner: Playlist) -> Self {
        Self {
            inner,
            actions: Vec::with_capacity(5),
        }
    }

    fn render(&mut self, ui: &mut egui::Ui, dark_mode: bool, actions: &[actions::Action]) {
        ui.horizontal(|ui| {
            ui.add_space(constants::PADDING * 2.);
            ui.style_mut().override_text_style = Some(utils::font::body());
            render_action_menu(ui, actions, "▶", &mut self.actions);
            ui.colored_label(
                constants::get_text_color(dark_mode),
                self.inner.name.to_string(),
            );
        });
        ui.add_space(constants::PADDING);
    }

    fn render_compact(&self, ui: &mut egui::Ui, dark_mode: bool) {
        ui.horizontal(|ui| {
            ui.colored_label(
                constants::get_text_color(dark_mode),
                self.inner.name.to_string(),
            );
        });
    }

    fn fulfilled_actions(&mut self) -> Vec<actions::Action> {
        let ret = self.actions.clone();
        self.actions.clear();
        ret
    }

    fn unique_id(&self) -> u64 {
        self.inner.unique_id()
    }
}
